@@ -252,23 +252,27 @@ susie_get_posterior_samples = function (susie_fit, num_samples) {
 #'   statistics. When the number of variables included in the CS is
 #'   greater than this number, the CS variables are randomly subsampled.
 #'
-#' @importFrom crayon red
-#' 
+#' @param use_rfast Use the Rfast package for the purity calculations.
+#'   By default \code{use_rfast = TRUE} if the Rfast package is
+#'   installed.
+#'
+#'
 #' @export
 #'
 susie_get_cs = function (res, X = NULL, Xcorr = NULL, coverage = 0.95,
                          min_abs_corr = 0.5, dedup = TRUE, squared = FALSE,
-                         check_symmetric = TRUE, n_purity = 100) {
+                         check_symmetric = TRUE, n_purity = 100, use_rfast) {
   if (!is.null(X) && !is.null(Xcorr))
     stop("Only one of X or Xcorr should be specified")
-  if (check_symmetric){
+  if (check_symmetric) {
     if (!is.null(Xcorr) && !is_symmetric_matrix(Xcorr)) {
-      message(red("Xcorr is not symmetric; forcing Xcorr to be symmetric ",
-                  "by replacing Xcorr with (Xcorr + t(Xcorr))/2"))
+      warning_message("Xcorr is not symmetric; forcing Xcorr to be symmetric",
+                  "by replacing Xcorr with (Xcorr + t(Xcorr))/2")
       Xcorr = Xcorr + t(Xcorr)
       Xcorr = Xcorr/2
     }
   }
+
   null_index = 0
   include_idx = rep(TRUE,nrow(res$alpha))
   if (!is.null(res$null_index)) null_index = res$null_index
@@ -295,22 +299,24 @@ susie_get_cs = function (res, X = NULL, Xcorr = NULL, coverage = 0.95,
   claimed_coverage = claimed_coverage[include_idx]
 
   # Compute and filter by "purity".
+  if (missing(use_rfast))
+    use_rfast = requireNamespace("Rfast",quietly = TRUE)
   if (is.null(Xcorr) && is.null(X)) {
     names(cs) = paste0("L",which(include_idx))
     return(list(cs = cs,
                 coverage = claimed_coverage,
                 requested_coverage = coverage))
   } else {
-    purity <- NULL
+    purity = NULL
     for (i in 1:length(cs)) {
       if (null_index > 0 && null_index %in% cs[[i]])
-        purity <- rbind(purity,c(-9,-9,-9))
+        purity = rbind(purity,c(-9,-9,-9))
       else
-        purity <-
+        purity =
           rbind(purity,
-                matrix(get_purity(cs[[i]],X,Xcorr,squared,n_purity),1,3))
+            matrix(get_purity(cs[[i]],X,Xcorr,squared,n_purity,use_rfast),1,3))
     }
-    purity <- as.data.frame(purity)
+    purity = as.data.frame(purity)
     if (squared)
       colnames(purity) = c("min.sq.corr","mean.sq.corr","median.sq.corr")
     else
@@ -361,9 +367,7 @@ susie_get_cs = function (res, X = NULL, Xcorr = NULL, coverage = 0.95,
 #'
 #' @return A matrix of correlations between CSs, or the maximum
 #'   absolute correlation when \code{max = TRUE}.
-#' 
-#' @importFrom crayon red
-#' 
+#'
 #' @export
 #'
 get_cs_correlation = function (model, X = NULL, Xcorr = NULL, max = FALSE) {
@@ -373,8 +377,8 @@ get_cs_correlation = function (model, X = NULL, Xcorr = NULL, max = FALSE) {
   if (is.null(Xcorr) && is.null(X))
     stop("One of X or Xcorr must be specified")
   if (!is.null(Xcorr) && !is_symmetric_matrix(Xcorr)) {
-    message(red("Xcorr is not symmetric; forcing Xcorr to be symmetric ",
-                "by replacing Xcorr with (Xcorr + t(Xcorr))/2"))
+    warning_message("Xcorr is not symmetric; forcing Xcorr to be symmetric",
+                "by replacing Xcorr with (Xcorr + t(Xcorr))/2")
     Xcorr = Xcorr + t(Xcorr)
     Xcorr = Xcorr/2
   }
@@ -471,13 +475,16 @@ n_in_CS = function(res, coverage = 0.9) {
 # Subsample and compute min, mean, median and max abs corr.
 #
 #' @importFrom stats median
-get_purity = function(pos, X, Xcorr, squared = FALSE, n = 100) {
-  if (requireNamespace("Rfast",quietly = TRUE)) {
-    get_upper_tri <- Rfast::upper_tri
-    get_median    <- Rfast::med
+get_purity = function (pos, X, Xcorr, squared = FALSE, n = 100,
+                       use_rfast) {
+  if (missing(use_rfast))
+    use_rfast = requireNamespace("Rfast",quietly = TRUE)
+  if (use_rfast) {
+    get_upper_tri = Rfast::upper_tri
+    get_median    = Rfast::med
   } else {
-    get_upper_tri <- upper.tri
-    get_median    <- stats::median
+    get_upper_tri = function (R) R[upper.tri(R)]
+    get_median    = stats::median
   }
   if (length(pos) == 1)
     return(c(1,1,1))
@@ -574,8 +581,8 @@ susie_prune_single_effects = function (s,L = 0,V = NULL) {
   if (L > num_effects) {
     message(paste("Specified number of effects L =",L,
                   "is greater the number of effects",num_effects,
-                  "in input SuSiE model. The SuSiE model will be expanded to have",L,
-                  "effects."))
+                  "in input SuSiE model. The SuSiE model will be expanded",
+                  "to have",L,"effects."))
 
     s$alpha = rbind(s$alpha[effects_rank,],
                     matrix(1/ncol(s$alpha),L - num_effects,ncol(s$alpha)))
@@ -618,6 +625,8 @@ susie_prune_single_effects = function (s,L = 0,V = NULL) {
 #' @param R A p by p symmetric, positive semidefinite correlation
 #'   matrix.
 #'
+#' @param n The sample size. (Optional, but highly recommended.)
+#'
 #' @param r_tol Tolerance level for eigenvalue check of positive
 #'   semidefinite matrix of R.
 #'
@@ -637,24 +646,44 @@ susie_prune_single_effects = function (s,L = 0,V = NULL) {
 #' input_ss = compute_suff_stat(X,y,standardize = TRUE)
 #' ss = univariate_regression(X,y)
 #' R = cor(X)
-#' attr(R,"eigen") = eigen(R, symmetric = TRUE)
+#' attr(R,"eigen") = eigen(R,symmetric = TRUE)
 #' zhat = with(ss,betahat/sebetahat)
-#' s = estimate_s_rss(zhat, R)
+#'
+#' # Estimate s using the unadjusted z-scores.
+#' s0 = estimate_s_rss(zhat,R)
+#' 
+#' # Estimate s using the adjusted z-scores.
+#' s1 = estimate_s_rss(zhat,R,n)
 #'
 #' @importFrom stats dnorm
 #' @importFrom stats optim
 #'
 #' @export
 #'
-estimate_s_rss = function (z, R, r_tol = 1e-08, method = "null-mle") {
+estimate_s_rss = function (z, R, n, r_tol = 1e-08, method = "null-mle") {
+
+  # Check and process input arguments z, R.
+  z[is.na(z)] = 0
   if (is.null(attr(R,"eigen")))
     attr(R,"eigen") = eigen(R,symmetric = TRUE)
   eigenld = attr(R,"eigen")
-  if(any(eigenld$values < -r_tol))
-    warning("The matrix R is not positive semidefinite. Negative ",
+  if (any(eigenld$values < -r_tol))
+    warning_message("The matrix R is not positive semidefinite. Negative ",
             "eigenvalues are set to zero")
   eigenld$values[eigenld$values < r_tol] = 0
 
+  # Check input n, and adjust the z-scores if n is provided.
+  if (missing(n))
+    warning_message("Providing the sample size (n), or even a rough estimate of n, ",
+            "is highly recommended. Without n, the implicit assumption is ",
+            "n is large (Inf) and the effect sizes are small (close to zero).")
+  else if (n <= 1)
+    stop("n must be greater than 1")
+  if (!missing(n)) {
+    sigma2 = (n-1)/(z^2 + n - 2)
+    z = sqrt(sigma2) * z
+  }
+
   if (method == "null-mle") {
     negloglikelihood = function(s, z, eigenld)
       0.5 * sum(log((1-s)*eigenld$values + s)) +
@@ -707,6 +736,8 @@ estimate_s_rss = function (z, R, r_tol = 1e-08, method = "null-mle") {
 #' @param R A p by p symmetric, positive semidefinite correlation
 #'   matrix.
 #'
+#' @param n The sample size. (Optional, but highly recommended.)
+#'
 #' @param r_tol Tolerance level for eigenvalue check of positive
 #'   semidefinite matrix of R.
 #'
@@ -743,50 +774,65 @@ estimate_s_rss = function (z, R, r_tol = 1e-08, method = "null-mle") {
 #' y = drop(X %*% beta + rnorm(n))
 #' ss = univariate_regression(X,y)
 #' R = cor(X)
-#' attr(R,"eigen") = eigen(R, symmetric = TRUE)
+#' attr(R,"eigen") = eigen(R,symmetric = TRUE)
 #' zhat = with(ss,betahat/sebetahat)
-#' cond_dist = kriging_rss(zhat, R)
+#' cond_dist = kriging_rss(zhat,R,n = n)
 #' cond_dist$plot
 #'
 #' @export
 #'
-kriging_rss = function (z, R, r_tol = 1e-08,
-                        s = estimate_s_rss(z,R,r_tol,method = "null-mle")) {
+kriging_rss = function (z, R, n, r_tol = 1e-08,
+                        s = estimate_s_rss(z,R,n,r_tol,method = "null-mle")) {
+
+  # Check and process input arguments z, R.
+  z[is.na(z)] = 0
   if (is.null(attr(R,"eigen")))
     attr(R,"eigen") = eigen(R,symmetric = TRUE)
   eigenld = attr(R,"eigen")
   if (any(eigenld$values < -r_tol))
-    warning("The matrix R is not positive semidefinite. Negative ",
+    warning_message("The matrix R is not positive semidefinite. Negative ",
             "eigenvalues are set to zero.")
   eigenld$values[eigenld$values < r_tol] = 0
 
+  # Check and progress input argument s.
+  force(s)
   if (s > 1) {
-    warning("The given s is greater than 1. We replace it with 0.8.")
+    warning_message("The given s is greater than 1. We replace it with 0.8.")
     s = 0.8
-  }
-  if (s < 0)
+  } else if (s < 0)
     stop("The s must be non-negative")
-
-  dinv = 1/((1-s)*eigenld$values + s)
+  
+  # Check input n, and adjust the z-scores if n is provided.
+  if ((!missing(n)) && (n <= 1))
+    stop("n must be greater than 1")
+  if (missing(n))
+    warning_message("Providing the sample size (n), or even a rough estimate of n, ",
+            "is highly recommended. Without n, the implicit assumption is ",
+            "n is large (Inf) and the effect sizes are small (close to zero).")
+  else {
+    sigma2 = (n-1)/(z^2 + n - 2)
+    z = sqrt(sigma2) * z
+  }
+  
+  dinv = 1/((1-s) * eigenld$values + s)
   dinv[is.infinite(dinv)] = 0
   precision = eigenld$vectors %*% (t(eigenld$vectors) * dinv)
   condmean = rep(0,length(z))
   condvar = rep(0,length(z))
-  for(i in 1:length(z)){
+  for (i in 1:length(z)) {
     condmean[i] = -(1/precision[i,i]) * precision[i,-i] %*% z[-i]
-    condvar[i] = 1/precision[i,i]
+    condvar[i]  = 1/precision[i,i]
   }
   z_std_diff = (z-condmean)/sqrt(condvar)
 
   # obtain grid
   a_min = 0.8
-  if (max(z_std_diff^2) < 1){
+  if (max(z_std_diff^2) < 1)
     a_max = 2
-  }else{
+  else
     a_max = 2*sqrt(max(z_std_diff^2))
-  }
   npoint = ceiling(log2(a_max/a_min)/log2(1.05))
-  a_grid = 1.05^((-npoint):0) * a_max
+  a_grid = 1.05^(seq(-npoint,0)) * a_max
 
   # compute likelihood
   sd_mtx      = outer(sqrt(condvar),a_grid)
@@ -797,29 +843,30 @@ kriging_rss = function (z, R, r_tol = 1e-08,
   # estimate weight
   w = mixsqp(matrix_llik,log = TRUE,control = list(verbose = FALSE))$x
 
-  logl0mix    = as.numeric(log(exp(matrix_llik) %*% w)) + lfactors
-  matrix_llik = dnorm(z + condmean, sd=sd_mtx, log = TRUE)
+  # Compute denominators in likelihood ratios.
+  logl0mix = drop(log(exp(matrix_llik) %*% (w + 1e-15))) + lfactors
+
+  # Compute numerators in likelihood ratios.
+  matrix_llik = dnorm(z + condmean,sd = sd_mtx,log = TRUE)
   lfactors    = apply(matrix_llik,1,max)
   matrix_llik = matrix_llik - lfactors
-  logl1mix    = as.numeric(log(exp(matrix_llik) %*% w)) + lfactors
-  logLRmix    = logl1mix - logl0mix
+  logl1mix    = drop(log(exp(matrix_llik) %*% (w + 1e-15))) + lfactors
+
+  # Compute (log) likelihood ratios.
+  logLRmix = logl1mix - logl0mix
 
   res = data.frame(z = z,condmean = condmean,condvar = condvar,
                    z_std_diff = z_std_diff,logLR = logLRmix)
 
   p = ggplot(res) + geom_point(aes(y = z, x = condmean)) +
-    labs(y = "Observed z scores", x = "Expected value") +
-    geom_abline(intercept = 0, slope = 1) +
-    theme_bw()
-
+        labs(y = "Observed z scores", x = "Expected value") +
+        geom_abline(intercept = 0, slope = 1) +
+        theme_bw()
   idx = which(logLRmix > 2 & abs(z) > 2)
-  if(length(idx) > 0) {
+  if (length(idx) > 0)
     p = p + geom_point(data = res[idx,],
                        aes(y = z, x = condmean),col = "red")
-  }
-
-  return(list(plot = p,
-              conditional_dist = res))
+  return(list(plot = p,conditional_dist = res))
 }
 
 # Compute the column means of X, the column standard deviations of X,
@@ -928,3 +975,18 @@ check_projection = function (A, b) {
   else
     return(list(status = FALSE,msg = msg))
 }
+
+# @title Utility function to display warning messages as they occur
+# @param ... warning message
+# @param style either "warning" or "hint"
+#'@importFrom crayon combine_styles
+warning_message = function(..., style=c("warning", "hint")) {
+  style = match.arg(style)
+  if (style=="warning" && getOption("warn")>=0) {
+    alert <- combine_styles("bold", "underline", "red")	
+    message(alert("WARNING:"), " ", ...)
+  } else {
+    alert <- combine_styles("bold", "underline", "magenta")	
+    message(alert("HINT:"), " ", ...)
+  }
+}
